function [ydot, yinit, option] = hhmodel_multi(t, y, flag)
% HH: Hodgkin-Huxley (1952) model of squid giant axon

global COMP
global st_c
global st_t
global Istim
global last_t
switch flag
    case ''
        % Model parameters ------------------------------------------------
        % Radius of HH (cm)
        r    = 1.0/20000.0;
        % Length of each HH (cm)
        l    = 1.0/10.0/COMP;
        % Surface area (cm^2)
        S_s  = pi * 2.0 * r * l;
        % Cross section (cm^2)
        S_c  = pi * r * r;
        % Resistivity (orm*cm)
        pRAx = 100.0;
        % Resistance (korm)
        RAx  = pRAx * l / S_c / 1000.0;
        % Axonal conductance (mS)
        gAx  = 1.0 / RAx;
        % Membrane capacitance (nF/cm^2 * cm^2)
        Cm   = 1.0   * S_s;
        % Maximum conductances (uS/cm^2 * cm^2)
        gNa  = 120.0 * S_s;    % sodium conductance
        gK   = 36.0  * S_s;     % potassium conductance
        gL   = 0.3   * S_s;      % leakage conductance
        % reversal potentials (mV)
        ENa  = 50.0;     % sodium
        EK   = -77.0;    % potassium
        EL   = -54.4;    % leak
        % Other
        ydot = [];
        
        % Simulation status -----------------------------------------------
        per = max(t) / last_t * 100;
        if st_t < per
            st_t = st_t + 1;
            bar = [];
            for i = 10:10:100
                if i < per
                    bar = [bar '='];
                else
                    bar = [bar ' '];
                end
            end
            fprintf('\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b')
            fprintf('[%s]%3d%% ', bar, floor(per));
        end
        % Compartment loop
        for comp = 1:1:COMP
            % State variables ---------------------------------------------
            
            V = y(4*(comp-1)+1); % Membrane potential (mV)
            m = y(4*(comp-1)+2); % Sodium current activation variable
            h = y(4*(comp-1)+3); % Sodium current inactivation variable
            n = y(4*(comp-1)+4); % Potassium current activation variable
            if comp ~= 1         % Membrane potential of other compartment
                Vbefo = y(4*(comp-2)+1); end
            if comp ~= COMP
                Vnext = y(4*(comp-0)+1); end
            % Input current (uA) ------------------------------------------
            if comp == st_c
                % Linear interpolation
                Iext = interp1(Istim(:,1), Istim(:,2), t);
            else Iext = 0.0;
            end
            % Activation and inactivation rates ---------------------------
            am = -0.1 * (V + 40.0) / (exp(-(V + 40.0) / 10.0) - 1.0);
            bm = 4.0 * exp(-(V + 65.0) / 18.0);
            ah = 0.07 * exp(-(V + 65.0) / 20.0);
            bh = 1.0 / (exp(-(V + 35.0) / 10.0) + 1.0);
            an = -0.01 * (V + 55.0) / (exp(-(V + 55.0) / 10.0) - 1.0);
            bn = 0.125 * exp(-(V + 65.0) / 80.0);
            % Current between each compartment ----------------------------
            if comp == 1
                Icomp = gAx * (Vnext - V);
            elseif comp == COMP
                Icomp = gAx * (Vbefo - V);
            else
                Icomp = gAx * (Vnext + Vbefo - 2.0 * V);
            end
            % ydot --------------------------------------------------------
            ydot = [ydot;
                (Iext + Icomp - gNa * m^3 * h * (V - ENa) - gK * n^4 * (V - EK) - gL * (V - EL)) / Cm;
                am * (1 - m) - bm * m;
                ah * (1 - h) - bh * h;
                an * (1 - n) - bn * n;];
        end
    case 'init'     % Initialization
        ydot = Istim(1, 1):0.1:Istim(end, 1);       % Time span
        yinit = [];
        for i = 1:1:COMP
            yinit = [yinit; -65.0; 0.0529; 0.5961; 0.3177];    % Initial v, m, h, n
        end
        option = odeset('MaxStep', 0.2);             % Option
end
